########### Data pre-processing ##########
#%%
import pandas as pd
from packaging import version

commits = pd.read_csv('../data/commits.csv')
df = pd.read_csv('../data/parsed_git_diff.csv')

"""
All diffs come from package.json file
repo_name, hash, change_type, line_number, added, deleted
If NaN --> nothing added/deleted. 
change_type refers to the type of modification on the file. 

Let (added, deleted) be a tuple of the added/deleted content in the file
(NaN, NaN) --> Modification type = NONE (IGNORE)
(X, NaN) --> Modification type = ADD (IGNORE)
(NaN, X) --> Modification type = DELETE (also considered MODIFY)
(X_v2, X_v1) --> Modification type = MODIFY if done on the same line --> dependency update

We only care about DELETE and MODIFY types because they some kind of "maintenance" on a dependency. 

In the case of (NaN X), we need to find if X is a dependency.

In the case of (X_v2, X_v1), we need to find if X is a dependency, if X_v2 has the same name as X_v1, 
and if the modification was done on the same line (dependency update)
Example ("uglify": "1.2", "uglify": "1.1")
For soundness, we should also find if v2 > v1 (updated version)
"""

#Remove extra whitespace and characters 
df['added'] = df['added'].str.replace(' ', '')
df['deleted'] = df['deleted'].str.replace(' ', '')
df['added'] = df['added'].str.rstrip(',')
df['deleted'] = df['deleted'].str.rstrip(',')

#Filter out single character in DELETED ex '}' or '}]'
df = df[(df['deleted'].str.len() > 2)]

#Split columns to get dependency name and version (if not a dep, we don't care since we will cross-ref)
df['added_dependency'] = df['added'].str.split(':').str[0]
df['added_version'] = df['added'].str.split(':').str[1]

df['deleted_dependency'] = df['deleted'].str.split(':').str[0]
df['deleted_version'] = df['deleted'].str.split(':').str[1]

#Remove " " characters un added_dep, deleted_dep, etc columns
df['added_dependency'] = df['added_dependency'].str.strip('"')
df['deleted_dependency'] = df['deleted_dependency'].str.strip('"')
df['added_version'] = df['added_version'].str.strip('"')
df['deleted_version'] = df['deleted_version'].str.strip('"')

#Adding change type for dependency/line, and not just modification on the file
#https://packaging.pypa.io/en/latest/version.html#packaging.version.parse 
def get_maintenance_type(row):
    if str(row['added']) == 'nan' and str(row['deleted']) == 'nan': 
        return 'none'
    if str(row['added']) != 'nan' and str(row['deleted']) == 'nan':
        return 'add'
    if (str(row['added']) == 'nan' and str(row['deleted']) != 'nan'):
        return 'delete'
    else:
        if (version.parse(str(row['added_version'])) > version.parse(str(row['deleted_version']))) and str(row['added_dependency']) == str(row['deleted_dependency']):
                return 'update'
        if (version.parse(str(row['added_version'])) < version.parse(str(row['deleted_version']))) and str(row['added_dependency']) == str(row['deleted_dependency']):
                return 'downgrade'
        if (version.parse(str(row['added_version'])) == version.parse(str(row['deleted_version']))) and str(row['added_dependency']) == str(row['deleted_dependency']):
            return 'none'
        else:
            return 'unknown'

df['maintenance'] = df.apply(lambda x: get_maintenance_type(x), axis=1)

#Verify if elements are shuffled instead of replaced
res = df.groupby(['repo_name', 'hash']).agg(tuple).applymap(list).reset_index()
res['shuffled'] = [(set(a) & set(b)) if len((set(a) & set(b))) != 0 else 'NaN' for a, b in zip(res.added, res.deleted)]
res = res[['repo_name', 'hash', 'shuffled']]

res_final = pd.merge(df, res, on=['repo_name', 'hash'])

def is_it_shuffled(row):
    if str(row['deleted']) in str(row['shuffled']) and str(row['maintenance']) != 'none':
        return 'shuffle'
    elif str(row['added_dependency']) != str(row['deleted_dependency']) and str(row['deleted']) != 'nan':
        return 'delete'
    else:
        return str(row['maintenance'])

res_final['maintenance_type'] = res_final.apply(lambda x: is_it_shuffled(x), axis=1)
res_final = res_final.drop(columns=['maintenance', 'shuffled'])

#Load in dependencies
deps = pd.read_csv('../data/npm_ls_dependencies.csv')
deps = deps.groupby(["repo_name", "dependency_name", "version"]).agg(
        {"depth": "min", "path": "count", "is_dev": "first", "is_runtime": "first"}).reset_index()

#Get bundled dependencies
bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

#Merge all deps with bundled deps so we have "bundled" column for all dep 
merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

"""
(X, NaN) - add
(NaN, X) - delete
((X1, V2), (X1, V1)) - update
((X1, V1), (X1, V2)) - downgrade
((X1, V1), (X2, V2)) - delete IF not shuffled 
((X1, V1), (X1, V1)) - none 

"""

deps_diffs_merged = pd.merge(merged, res_final, left_on=['repo_name', 'dependency_name'], right_on=['repo_name', 'deleted_dependency'])
deps_diffs_merged['deleted_version'] = deps_diffs_merged['deleted_version'].str.strip('^')
deps_diffs_merged['added_version'] = deps_diffs_merged['added_version'].str.strip('^')

deps_diffs_final = deps_diffs_merged[((deps_diffs_merged['version'] == deps_diffs_merged['deleted_version']) & 
    (deps_diffs_merged['dependency_name'] == deps_diffs_merged['deleted_dependency'])) | 
    ((deps_diffs_merged['version'] == deps_diffs_merged['added_version']) & (deps_diffs_merged['dependency_name'] == deps_diffs_merged['added_dependency']))]


deps_diffs_final = pd.merge(commits, deps_diffs_final, on=['repo_name', 'hash'])

#Getting commit messages
deps_diffs_final = deps_diffs_final[['repo_name','dependency_name','version','depth','path','is_dev','is_runtime','is_bundled','hash','change_type',
'line_number','added','deleted','added_dependency','added_version','deleted_dependency','deleted_version','maintenance_type', 'msg']]

deps_diffs_final = deps_diffs_final.drop_duplicates()

#Remove "shuffle" because not a maintenance action
deps_diffs_final = deps_diffs_final[deps_diffs_final['maintenance_type'] != 'shuffle']

deps_diffs_final.to_csv('../data/deps_and_diffs.csv', mode='w', index=False)

########### Analysis ###########
# %%
import pandas as pd

df = pd.read_csv('../data/deps_and_diffs.csv')
#Remove 'none'
df = df[df['maintenance_type'] != 'none']

#How many total unique dependencies? 
unique_deps = df.groupby(["repo_name", "dependency_name", "version"]).agg({"is_bundled": "first", "is_runtime": "first", "is_dev": "first"}).reset_index()

#How many repos?
repos = df.groupby(["repo_name"]).count().reset_index()

#How many deleted/update/downgrade/replace actions?
action = len(df[df['maintenance_type'] == 'delete'])

#How many maintenance tasks for bundled/unbundled?
bundled = len(df[df['is_bundled'] == True])
unbundled = len(df[df['is_bundled'] == False])

#How many maintenance tasks for dev/runtime deps?
dev = len(df[df['is_dev'] == True])
runtime = len(df[df['is_runtime'] == True])

#How many maintenance tasks are for development AND bundled?
bundled_dev = len(df[(df['is_dev'] == True) & (df['is_bundled'] == True)])

#Maintenance type per bundled/unbundled
action_bundled = len(df[(df['is_bundled'] == True) & (df['maintenance_type'] == 'delete')])
action_unbundled = len(df[(df['is_bundled'] == False) & (df['maintenance_type'] == 'delete')])

# %%
import pandas as pd
deps = pd.read_csv('../data/npm_ls_dependencies.csv')
deps = deps.groupby(["repo_name", "dependency_name", "version"]).agg(
        {"depth": "min", "path": "count", "is_dev": "first", "is_runtime": "first"}).reset_index()

#Get bundled dependencies
bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

#Merge all deps with bundled deps so we have "bundled" column for all dep 
merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

# %%
import pandas as pd

df = pd.read_csv('../data/deps_and_diffs.csv')
#Remove 'none'
df = df[df['maintenance_type'] != 'none']
maintained_deps = df.groupby(["repo_name", "dependency_name"]).size().to_frame('nb_times_maintained').reset_index()
# %%
