# %%
import seaborn as sns
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

"""
Generate figures for the paper
"""

# %%
def plot_stacked_bar(data, series_labels, category_labels=None, 
                     show_values=False, value_format="{}", y_label=None, 
                     colors=None, grid=False, reverse=False):
    """Plots a stacked bar chart with the data and labels provided.

    Keyword arguments:
    data            -- 2-dimensional numpy array or nested list
                       containing data for each series in rows
    series_labels   -- list of series labels (these appear in
                       the legend)
    category_labels -- list of category labels (these appear
                       on the x-axis)
    show_values     -- If True then numeric value labels will 
                       be shown on each bar
    value_format    -- Format string for numeric value labels
                       (default is "{}")
    y_label         -- Label for y-axis (str)
    colors          -- List of color labels
    grid            -- If True display grid
    reverse         -- If True reverse the order that the
                       series are displayed (left-to-right
                       or right-to-left)
    """

    ny = len(data[0])
    ind = list(range(ny))

    axes = []
    cum_size = np.zeros(ny)

    data = np.array(data)

    if reverse:
        data = np.flip(data, axis=1)
        category_labels = reversed(category_labels)

    for i, row_data in enumerate(data):
        color = colors[i] if colors is not None else None
        axes.append(plt.bar(ind, row_data, bottom=cum_size, 
                            label=series_labels[i], color=color))
        cum_size += row_data

    if category_labels:
        plt.xticks(ind, category_labels)

    if y_label:
        plt.ylabel(y_label)

    plt.legend()

    if grid:
        plt.grid()

    if show_values:
        for axis in axes:
            for bar in axis:
                w, h = bar.get_width(), bar.get_height()
                plt.text(bar.get_x() + w/2, bar.get_y() + h/2, 
                         value_format.format(h), ha="center", 
                         va="center")

series_labels = ['Transitive', 'Direct']

data = [
    [675, 46673],
    [220, 1627]
]

category_labels = ['Runtime', 'Development']

plot_stacked_bar(
    data, 
    series_labels, 
    category_labels=category_labels, 
    show_values=True, 
    value_format="{:.1f}",
    colors=['#CD5C5C', '#6495ED'],
    y_label="Count"
)
plt.ylim(1,1000000)  # set y axis limits
plt.yscale('log')
plt.xlabel('Types of dependencies')
plt.show()

### DEPTH FOR PROJECTS THAT SIHP SOMETIHNG ###
# %%
dps_df = pd.read_csv('../data/npm_ls_dependencies.csv')
dps_df.loc[:, 'root'] = dps_df.path.map(lambda x: x.split("'")[1])
dps_df = dps_df.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

bundled_deps = dps_df.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
bundled_deps = dps_df.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
bundled_deps.fillna(False, inplace=True)
bundled_deps = bundled_deps.drop_duplicates()

# Projects that ship something
projects_ship = bundled_deps[bundled_deps["repo_name"].isin(bundled_df["repo_name"].unique())]
projects_ship = projects_ship.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first", "is_bundled": "first"}).reset_index()

bundled_depth = projects_ship[projects_ship["is_bundled"] == True]["depth"]
unbundled_depth = projects_ship[projects_ship["is_bundled"] == False]["depth"]

#data = np.concatenate([[bundled_depth, len(bundled_depth)*['bundled'], len(bundled_depth)*['bundled']], [unbundled_depth, len(unbundled_depth)*['unbundled'],len(unbundled_depth)*['unbundled']]],axis=1)
data = np.concatenate([[unbundled_depth, len(unbundled_depth)*['unbundled'],len(unbundled_depth)*['unbundled']], [bundled_depth, len(bundled_depth)*['bundled'], len(bundled_depth)*['bundled']]],axis=1)

df = pd.DataFrame(columns=['depth', 'dependency', 'dependency label'], data=data.T)
df['depth'] = df['depth'].astype(int)
sns.set_style('whitegrid')
ax = sns.boxplot(x='depth', y='dependency', hue='dependency label', linewidth=1, orient="h", data=df, palette={'bundled': '0.8', 'unbundled': 'b'})
ax.set(yticklabels=[])
ax.set(ylabel=None) 

plt.savefig('../figures/bundled_projects_depth.pdf')
plt.show()

# %%
#sns.set_style('whitegrid')
sns.set_context('paper', font_scale=1.2)

dps_df = pd.read_csv('../data/npm_ls_dependencies.csv')
dps_df.loc[:, 'root'] = dps_df.path.map(lambda x: x.split("'")[1])
dps_df = dps_df.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

bundled_deps = dps_df.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
bundled_deps.fillna(False, inplace=True)

# Projects that ship something
projects_ship = bundled_deps[bundled_deps["repo_name"].isin(bundled_df["repo_name"].unique())]
projects_ship = projects_ship.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first", "is_bundled": "first"}).reset_index()

bundled_count = bundled_df.groupby(['repo_name']).size().to_frame('count').reset_index()

# ax = sns.catplot(data=bundled_count, x="count", kind="count", palette="Blues_r")


#%%
sns.set_context('paper', font_scale=1.3)
to_plot = pd.cut(bundled_count['count'], bins=[0, 1, 5, 10, 50, 100]).value_counts()

ax = sns.barplot(x=to_plot.index, y=to_plot, palette="Blues_r")

ax.set_xticklabels(['1', '2-5', '6-10', '11-50', '50+'])
ax.set(xlabel='Number of Production Dependencies', ylabel='Number of Projects')

sns.despine()

plt.savefig('../figures/bundled_dep_per_project.pdf')
plt.show()

# Bundled, dev dependencies - root package
#https://stackabuse.com/seaborn-bar-plot-tutorial-and-examples/
# %%
#sns.set_style('whitegrid')
sns.set_context('paper', font_scale=1.2)

x = ['webpack', '@babel/preset-env', 'react-live', 'react-dom', 'react-router']
y = [17, 16, 14, 7, 7]

df = pd.DataFrame({'Root dependencies': x, 'Frequency': y})

#sns.barplot(x, y, palette="Blues_r")
sns.barplot(data=df, y="Frequency", x="Root dependencies", palette="Blues_r", )
sns.despine()
plt.xticks(rotation=30)
plt.savefig('../figures/most_frequent_root.pdf')
plt.show()

# %%
import seaborn as sns
import pandas as pd
import matplotlib.pyplot as plt

unique_deps = pd.read_csv('../data/unique_deps_rq2.csv')

#Always in prod
always_prod = unique_deps[(unique_deps['installed_prod'] != 0) & (unique_deps['installed_notprod'] == 0)]

#Never in prod
never_prod = unique_deps[(unique_deps['installed_notprod'] != 0) & (unique_deps['installed_prod'] == 0)]

#Sometimes in prod, sometimes not in prod
in_between = unique_deps[(unique_deps['installed_notprod'] != 0) & (unique_deps['installed_prod'] != 0)]

sns.boxplot(data=in_between, x='installed_prod')
#sns.set_context('paper', font_scale=1.3)

#ax = sns.boxplot(x="variable", y="value", data=pd.melt(unique_deps))
plt.show()

# %%
import seaborn as sns
import pandas as pd
import matplotlib.pyplot as plt

data=[140, 1086, 43]
d={'Always in prod': 140, 'Never in prod': 1086, 'In between': 43}
df = pd.DataFrame(d, index=[0])
sns.boxplot(data=pd.melt(df), x='variable', y='value', orient='v')
# %%
