import json
import pandas as pd
import re
from dataclasses import dataclass
from json import JSONDecodeError

"""
This script was modified from the original source 
https://github.com/unisil/source-map-parser 

Extracts sourcemaps files source contents to get the node 
module packages bundled by webpack or rollup. 
"""

@dataclass
class SourceMap:
    version: int
    sources: str
    sources_content: str


def read_local_resource(path):
    try:
        with open(path, "r", encoding="utf-8") as input_file:
            return json.load(input_file)

    except (FileNotFoundError, JSONDecodeError) as e:
        if e.__class__.__name__ == "FileNotFoundError":
            exit(f"[{path}] Provided JSON path doesn't exist, check and try again")

        elif e.__class__.__name__ == "JSONDecodeError":
            exit(
                f"[{path}] Provided JSON has not been parsed as valid JSON, check and try again"
            )

def verify_source_map(source_map):
    keys = ["version", "sources", "sourcesContent"]
    return all([key in source_map for key in keys])

def make_clean_path(path):
    invalid = '^*":<>|?'

    path = path.replace(" ^\.\/.*$", ".js")

    for character in invalid:
        path = path.replace(character, "")

    return path

def parse(resource, destination_file):
    print(f"[{resource}] Parsing local resource")

    source_map = read_local_resource(resource)

    if verify_source_map(source_map):
        source_map = SourceMap(
            version=int(source_map["version"]),
            sources=source_map["sources"],
            sources_content=source_map["sourcesContent"],
        )
        print(
            f"[{resource}] Version {source_map.version} source map with {len(source_map.sources)} sources"
        )
        with open(destination_file, 'w') as f:
            for source in source_map.sources:
                f.write(source)
                f.write('\n')
    else:
        with open(destination_file, 'w') as f:
            pass
        
def get_dependencies(input_path, output_path):
    deps = []
    result = ''
    with open(input_path, 'r') as f:
        lines = f.readlines()
        for line in lines:
            if 'node_modules' in line:
                if '@' in line:
                    result = re.search(r"(?:node_modules\/)(.*)", line).group(1)
                    result = result.split('/')[1]
                    deps.append(result)
                else:
                    result = re.search(r"(?:node_modules\/)(.*)", line).group(1)
                    result = result.split('/')[0]
                    deps.append(result)
        deps = list(dict.fromkeys(deps))

    with open(output_path, 'w') as f:
        for dep in deps:
            f.write(dep)
            f.write('\n')

if __name__ == "__main__":
    
    df = pd.read_csv('../data/dataset.csv',converters={"map_files": lambda x: x.strip("[]").replace("'","").split(", ")})

    repo_paths = df['repo_path']
    sourcemaps = df['map_files']

    for path in repo_paths:
        sourcemaps = df.loc[df['repo_path'] == path, 'map_files']
        for sourcemap in sourcemaps:
            for s in sourcemap:
                parse(s, path + '/unpack.txt')

    for path in repo_paths:
        try:
            input_file = path + '/unpack.txt'
            output_file = path + '/bundled-dependencies.txt'

            get_dependencies(input_file, output_file)
        except:
            print('Nothing to unpack in ' + path)
            pass

